module SecureScope
  module ControllerInitializer
    class ActionController::Base
      def self.secure_scope(scopes_hash)
        scopes_hash.keys.each do |action_key|
          method_builder(scopes_hash, action_key) if scopes_hash[action_key]
        end
      end

      def self.method_builder(scopes_hash, action_key)
        define_method "#{action_key.to_s}_secure_scope" do
          scopes_hash[action_key].keys.each do |permission_key|
            if user_type_match?(permission_key)
              return scope_builder scopes_hash[action_key][permission_key]
            end
          end
          raise NotAllowed
        end
      end

      def user_type_match?(permission_key)
        current_authenticated.kind_of? get_model(permission_key)
      end

      def get_model(permission_key)
        Kernel.const_get(permission_key.to_s.camelize)
      end

      def scope_builder(scope)
        return scope unless scope.kind_of?(Proc)
        scope.call(current_authenticated)
      end

      class NotAllowed < Exception; end
      rescue_from NotAllowed do |e|
        render json: { error: e.message || "You're not authorized to view that." }, status: :forbidden
      end
    end
  end
end
